/******************************************************************************/
/* Piccola libreria per la gestione di dispositivi I2C                        */
/******************************************************************************/
#include <xc.h>
#include <i2c.h>
#include "header.h"


//Inizializza il modulo MSSP in modalit I2C master. La frequenza deve essere espressa in KHz
void I2C_init(int freq)
{
    unsigned long tmp;
    //Pin SDA e SCK del modulo I2C
    TRISB4=1;
    TRISB5=1;

    //Controlla se viene utilizzata una frequenza standard o meno, per attivare lo slew rate control
    if (freq==100 || freq==1000) SSP1STAT=0b10000000;
    else SSP1STAT=0;
    SSP1CON1=0b00101000;
    SSP1CON2=0;
    //Fout=Fosc/(4*(SPP1ADD+1)) ==> SPP1ADD=(Fosc/(4*Fout))-1
    tmp=_XTAL_FREQ/1000;
    tmp/=4;
    tmp/=freq;
    SSP1ADD=tmp-1;
}




//Invio segnale di start
void I2C_start(void)
{
    SSP1CON2bits.SEN=1; //Invia il segnale di start
    while(!SSP1IF);     //Attende che il segnale sia stato inviato
    SSP1IF=0;
}
//Invio un byte
void I2C_write(unsigned char dato)
{
    SSP1BUF=dato;       //Invio del dato via I2C
    while(!SSP1IF);     //Attende che il dato sia stato inviato
    SSP1IF=0;
}
//Lettura di un byte ricevuto
unsigned char I2C_read(unsigned char ack)
{
    unsigned dato;
    SSP1CON2bits.RCEN=1;             //Modalit ricezione
    while(!SSP1IF);                  //Attende che il dato sia stato ricevuto
    SSP1IF=0;
    dato=SSP1BUF;
    if (ack) SSP1CON2bits.ACKDT=1;   //Setto che deve inviare un segnale di acknowledge
    else SSP1CON2bits.ACKDT=0;       //Setto che deve inviare un segnale di not-acknowledge
    SSP1CON2bits.ACKEN=1;            //Invio il segnale
    while(!SSP1IF);                 //Attende che il segnale sia stato inviato
    SSP1IF=0;
    return dato;
}
//Ripetizione del segnale di start
void I2C_rep_start(void)
{
    SSP1CON2bits.RSEN=1;    //Ripeto il segnale di start
    while(!SSP1IF);     //Attende che il segnale sia stato inviato
    SSP1IF=0;
}
//Invio segnale di stop
void I2C_stop(void)
{
    SSP1CON2bits.PEN=1; //Segnale di stop
    while(!SSP1IF);     //Attende che il dato sia stato inviato
    SSP1IF=0;
}